@@ -686,6 +686,38 @@ repository(id: "repo1") {
 >>> obj.repository.name
 'Repo #1'
 
+
+Error Reporting
+~~~~~~~~~~~~~~~
+
+If the returned data contains only ``errors`` and no ``data``,
+the interpretation will raise an error :class:`GraphQLErrors`:
+
+>>> json_data = {'errors': [{'message': 'some message'}]}
+>>> try:  # doctest: +ELLIPSIS
+...     obj = op + json_data
+... except GraphQLErrors as ex:
+...     print('Got error:', repr(ex))
+...     print(ex.errors)
+Got error: GraphQLErrors('some message'...
+[{'message': 'some message'}]
+
+If there are mixed data and errors, the object is returned with
+``__errors__`` attribute set to the errors:
+
+>>> json_data = {
+...     'errors': [{'message': 'some message'}],
+...     'data': {
+...         'repository': {'id': 'repo1', 'name': 'Repo #1'},
+...     },
+... }
+>>> obj = op + json_data
+>>> obj.repository.name
+'Repo #1'
+>>> obj.__errors__
+[{'message': 'some message'}]
+
+
 Mutations
 ~~~~~~~~~
 
@@ -861,6 +893,152 @@ it's going to behave as the interface type as well:
 Actor(login='user')
 
 
+Named Fragments
+~~~~~~~~~~~~~~~
+
+Named fragments are a way to reuse selection blocks and allow
+optimizations to be employed. They also allow shorter documents
+if the fragment is used more than once.
+
+They are similar to Inline Fragments described above as they
+allow selecting on interfaces and unions.
+
+>>> org_loc_frag = Fragment(Organization, 'OrganizationLocationFragment')
+>>> org_loc_frag.location()
+location
+>>> org_login_frag = Fragment(Organization, 'OrganizationLoginFragment')
+>>> org_login_frag.login()
+login
+>>> user_frag = Fragment(User, 'UserFragment')
+>>> user_frag.name()
+name
+>>> assignee_frag = Fragment(Assignee, 'AssigneeFragment')
+>>> assignee_frag.email()
+email
+>>> op = Operation(Query)
+>>> repo = op.repository(id='repo1')
+>>> repo.owner.login() # interface fields can be declared as usual
+login
+>>> repo.owner().__fragment__(org_loc_frag)
+>>> repo.owner().__fragment__(org_login_frag)  # can do many on the same type
+>>> repo.owner.__fragment__(user_frag)
+>>> repo.issues().assigned.__fragment__(assignee_frag)
+>>> repo.issues().assigned.__fragment__(user_frag)
+>>> repo.issues().commenters().actors().login()
+login
+>>> repo.issues().commenters().actors().__fragment__(org_loc_frag)
+>>> repo.issues().commenters().actors().__fragment__(user_frag)
+>>> op
+query {
+  repository(id: "repo1") {
+    owner {
+      login
+      __typename
+      ...OrganizationLocationFragment
+      ...OrganizationLoginFragment
+      ...UserFragment
+    }
+    issues {
+      assigned {
+        __typename
+        ...AssigneeFragment
+        ...UserFragment
+      }
+      commenters {
+        actors {
+          login
+          __typename
+          ...OrganizationLocationFragment
+          ...UserFragment
+        }
+      }
+    }
+  }
+}
+fragment OrganizationLocationFragment on Organization {
+  location
+}
+fragment OrganizationLoginFragment on Organization {
+  login
+}
+fragment UserFragment on User {
+  name
+}
+fragment AssigneeFragment on Assignee {
+  email
+}
+
+Note that ``__typename`` is automatically selected so it can create the
+proper type when interprets the results:
+
+>>> json_data = {'data': {'repository': {'owner': {
+...    '__typename': 'User',
+...    'login': 'user',
+...    'name': 'User Name',
+...    },
+...    'issues': [
+...      {
+...          'assigned': {'__typename': 'Assignee', 'email': 'e@mail.com'},
+...          'commenters': {
+...              'actors': [
+...                  {'login': 'user', '__typename': 'User', 'name': 'User Name'},
+...                  {'login': 'a-company', '__typename': 'Organization', 'location': 'that place'}
+...              ]
+...          }
+...      },
+...      {
+...          'assigned': {'__typename': 'User', 'name': 'User'},
+...          'commenters': {
+...              'actors': [
+...                  {'login': 'user', '__typename': 'User', 'name': 'User Name'},
+...                  {'login': 'xpto', '__typename': 'User'}
+...              ]
+...          }
+...      },
+...    ],
+... }}}
+>>> obj = op + json_data
+>>> obj.repository.owner
+User(login='user', __typename__='User', name='User Name')
+>>> for i in obj.repository.issues:
+...     print(i)
+Issue(assigned=Assignee(__typename__=Assignee, email=e@mail.com), commenters=ActorConnection(actors=[User(login='user', __typename__='User', name='User Name'), Organization(login='a-company', __typename__='Organization', location='that place')]))
+Issue(assigned=User(__typename__=User, name=User), commenters=ActorConnection(actors=[User(login='user', __typename__='User', name='User Name'), User(login='xpto', __typename__='User')]))
+
+>>> json_data = {'data': {'repository': {'owner': {
+...    '__typename': 'Organization',
+...    'login': 'a-company',
+...    'location': 'somewhere',
+...    'name': 'A Company',
+... }}}}
+>>> obj = op + json_data
+>>> obj.repository.owner
+Organization(login='a-company', __typename__='Organization', location='somewhere')
+
+If the returned type doesn't have an explicit type fields, the
+Interface field is returned:
+
+>>> json_data = {'data': {'repository': {'owner': {
+...    '__typename': 'SomethingElse',
+...    'login': 'something-else',
+...    'field': 'value',
+... }}}}
+>>> obj = op + json_data
+>>> obj.repository.owner
+Actor(login='something-else', __typename__='SomethingElse')
+
+In the unusual situation where ``__typename`` is not returned,
+it's going to behave as the interface type as well:
+
+>>> json_data = {'data': {'repository': {'owner': {
+...    'login': 'user',
+...    'name': 'User Name',
+... }}}}
+>>> obj = op + json_data
+>>> obj.repository.owner
+Actor(login='user')
+
+
 Utilities
 ~~~~~~~~~
 
@@ -1038,6 +1216,9 @@ from ..types import BaseTypeWithTypename, Union, ContainerType, ArgDict, \
 
 
 DEFAULT_AUTO_SELECT_DEPTH = 2
+PROXIED_FIELDS = {
+    '__type__', '__casts__', '__as__', '__fragment__', '__fragments__',
+}
 
 
 class Selection:
@@ -1281,6 +1462,10 @@ class Selection:
                 indent, indent_string, auto_select_depth)
         return prefix + alias + self.__field__.graphql_name + args + query
 
+    def __collect_fragments__(self, fragments):
+        if self.__selection_list:
+            self.__selection_list.__collect_fragments__(fragments)
+
     def __dir__(self):
         original_dir = super(Selection, self).__dir__()
         t = self.__field__.type
@@ -1292,8 +1477,7 @@ class Selection:
     def __getattr__(self, name):
         if name.startswith('_'):
             sl = self.__selection_list
-            proxied_fields = ('__type__', '__casts__', '__as__')
-            if name in proxied_fields:
+            if name in PROXIED_FIELDS:
                 if sl is None:
                     return None
                 return getattr(sl, name)
@@ -1302,6 +1486,9 @@ class Selection:
             return self[name]
         except KeyError as exc:
             raise AttributeError('%s has no field %s' % (self, name)) from exc
+        except ValueError as exc:
+            raise AttributeError('%s failed to get field %s' %
+                                 (self, name)) from exc
 
     def __getitem__(self, name):
         if self.__selection_list is None:
@@ -1426,6 +1613,11 @@ class Selector:
         '''
         return self().__as__(typ)
 
+    def __fragment__(self, fragment):
+        '''Spread the given fragment in the selection list.
+        '''
+        return self().__fragment__(fragment)
+
     def __dir__(self):
         original_dir = super(Selector, self).__dir__()
         t = self.__field__.type
@@ -1456,6 +1648,9 @@ class Selector:
             return self[name]
         except KeyError as exc:
             raise AttributeError('%s has no field %s' % (self, name)) from exc
+        except ValueError as exc:
+            raise AttributeError('%s failed to get field %s' %
+                                 (self, name)) from exc
 
     def __getitem__(self, name):
         return self()[name]
@@ -1530,7 +1725,8 @@ class SelectionList:
 
     '''  # noqa: E501
 
-    __slots__ = ('__type', '__selectors', '__selections', '__casts')
+    __slots__ = ('__type', '__selectors', '__selections', '__casts',
+                 '__fragments')
 
     def __init__(self, typ):
         assert issubclass(typ, BaseTypeWithTypename), \
@@ -1539,6 +1735,7 @@ class SelectionList:
         self.__selectors = {}
         self.__selections = []
         self.__casts = OrderedDict()
+        self.__fragments = OrderedDict()
 
     def __str__(self):
         return self.__to_graphql__()
@@ -1565,6 +1762,9 @@ class SelectionList:
             s.append(next_prefix + v.__to_graphql__(
                 next_indent, indent_string, auto_select_depth,
             ))
+        for lst in self.__fragments.values():
+            for v in lst:
+                s.append(next_prefix + v.__spread__())
 
         s.append(prefix + '}')
         return '\n'.join(s)
@@ -1603,6 +1803,24 @@ class SelectionList:
     def __casts__(self):
         return self.__casts
 
+    @property
+    def __fragments__(self):
+        return self.__fragments
+
+    def __collect_fragments__(self, fragments=None):
+        if fragments is None:
+            fragments = OrderedDict()
+
+        for lst in self.__fragments.values():
+            for frag in lst:
+                fragments.setdefault(frag.__name__, frag)
+                frag.__collect_fragments__(fragments)
+
+        for sel in self.__selections:
+            sel.__collect_fragments__(fragments)
+
+        return fragments
+
     def __as__(self, typ):
         '''Create a child selection list on the given type.
 
@@ -1624,6 +1842,13 @@ class SelectionList:
         self['__typename__']()
         return sl
 
+    def __fragment__(self, fragment):
+        assert isinstance(fragment, Fragment)
+        self.__fragments.setdefault(
+            fragment.__type__.__name__, []
+        ).append(fragment)
+        self['__typename__']()
+
 
 class InlineFragmentSelectionList(SelectionList):
     def __to_graphql__(self, indent=0, indent_string='  ',
@@ -1633,6 +1858,36 @@ class InlineFragmentSelectionList(SelectionList):
         return '... on %s %s' % (self.__type__, selection)
 
 
+class Fragment(SelectionList):
+    __slots__ = ('__name',)
+
+    def __init__(self, typ, name):
+        super(Fragment, self).__init__(typ)
+        self.__name = name
+
+    @property
+    def __name__(self):
+        return self.__name
+
+    def __spread__(self):
+        return '...%s' % (self.__name,)
+
+    def __to_graphql__(self, indent=0, indent_string='  ',
+                       auto_select_depth=DEFAULT_AUTO_SELECT_DEPTH):
+        selection = SelectionList.__to_graphql__(
+            self, indent, indent_string, auto_select_depth)
+        return 'fragment %s on %s %s' % (
+            self.__name, self.__type__, selection)
+
+
+class GraphQLErrors(RuntimeError):
+    def __init__(self, errors):
+        assert len(errors) > 0
+        msg = str(errors[0].get('message'))
+        super(RuntimeError, self).__init__(msg)
+        self.errors = errors
+
+
 class Operation:
     '''GraphQL Operation: query or mutation.
 
@@ -1731,6 +1986,20 @@ class Operation:
       }
     }
 
+    If variable name conflicts with the parameter, you can pass
+    them as a single ``variables`` parameter containing a dict.
+
+    >>> from sgqlc.types import Variable
+    >>> op = Operation(name='MyOperation', variables={'name': str})
+    >>> op.repository(id=Variable('name')).name()
+    name
+    >>> op # or repr(), prints out GraphQL!
+    query MyOperation($name: String) {
+      repository(id: $name) {
+        name
+      }
+    }
+
     Complex argument types are also supported as JSON object (GraphQL names
     and raw types) or actual types:
 
@@ -1795,6 +2064,8 @@ class Operation:
             typ = global_schema.query_type
 
         variable_args = OrderedDict()
+        if len(args) == 1 and 'variables' in args:
+            args = args['variables']
         for k, v in args.items():
             variable_args['$' + k] = v
 
@@ -1835,7 +2106,17 @@ class Operation:
         args = self.__args.__to_graphql__(indent, indent_string)
         selections = self.__selection_list.__to_graphql__(
             indent, indent_string, auto_select_depth)
-        return prefix + kind + name + args + ' ' + selections
+
+        frags_gql = []
+        fragments = self.__selection_list.__collect_fragments__()
+        for fragment in fragments.values():
+            frags_gql.append(fragment.__to_graphql__(indent, indent_string,
+                             auto_select_depth))
+
+        frags = ''
+        if frags_gql:
+            frags = '\n' + '\n'.join(frags_gql)
+        return prefix + kind + name + args + ' ' + selections + frags
 
     def __iter__(self):
         return iter(self.__selection_list)
@@ -1862,4 +2143,16 @@ class Operation:
         return bytes(self.__to_graphql__(indent_string=''), 'utf-8')
 
     def __add__(self, other):
-        return self.__type(other.get('data'), self.__selection_list)
+        errors = other.get('errors')
+        ex = None
+        if errors:
+            ex = GraphQLErrors(errors)
+
+        data = other.get('data')
+        if not data:
+            if not ex:  # pragma: no cover
+                ex = ValueError('no data and no errors')
+            raise ex
+        o = self.__type(other.get('data'), self.__selection_list)
+        o.__errors__ = errors
+        return o
